# coding: utf-8

"""
    Google Docs API

    Reads and writes Google Docs documents.

    The version of the OpenAPI document: v1
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictStr, field_validator
from typing import Any, ClassVar, Dict, List, Optional
from google_docs_ti.models.body import Body
from google_docs_ti.models.document_style import DocumentStyle
from google_docs_ti.models.named_styles import NamedStyles
from google_docs_ti.models.tab import Tab
from typing import Optional, Set
from typing_extensions import Self

class Document(BaseModel):
    """
    A Google Docs document.
    """ # noqa: E501
    body: Optional[Body] = None
    document_id: Optional[StrictStr] = Field(default=None, description="Output only. The ID of the document.", alias="documentId")
    document_style: Optional[DocumentStyle] = Field(default=None, alias="documentStyle")
    footers: Optional[Dict[str, Any]] = Field(default=None, description="Output only. The footers in the document, keyed by footer ID. Legacy field: Instead, use Document.tabs.documentTab.footers, which exposes the actual document content from all tabs when the includeTabsContent parameter is set to `true`. If `false` or unset, this field contains information about the first tab in the document.")
    footnotes: Optional[Dict[str, Any]] = Field(default=None, description="Output only. The footnotes in the document, keyed by footnote ID. Legacy field: Instead, use Document.tabs.documentTab.footnotes, which exposes the actual document content from all tabs when the includeTabsContent parameter is set to `true`. If `false` or unset, this field contains information about the first tab in the document.")
    headers: Optional[Dict[str, Any]] = Field(default=None, description="Output only. The headers in the document, keyed by header ID. Legacy field: Instead, use Document.tabs.documentTab.headers, which exposes the actual document content from all tabs when the includeTabsContent parameter is set to `true`. If `false` or unset, this field contains information about the first tab in the document.")
    inline_objects: Optional[Dict[str, Any]] = Field(default=None, description="Output only. The inline objects in the document, keyed by object ID. Legacy field: Instead, use Document.tabs.documentTab.inlineObjects, which exposes the actual document content from all tabs when the includeTabsContent parameter is set to `true`. If `false` or unset, this field contains information about the first tab in the document.", alias="inlineObjects")
    lists: Optional[Dict[str, Any]] = Field(default=None, description="Output only. The lists in the document, keyed by list ID. Legacy field: Instead, use Document.tabs.documentTab.lists, which exposes the actual document content from all tabs when the includeTabsContent parameter is set to `true`. If `false` or unset, this field contains information about the first tab in the document.")
    named_ranges: Optional[Dict[str, Any]] = Field(default=None, description="Output only. The named ranges in the document, keyed by name. Legacy field: Instead, use Document.tabs.documentTab.namedRanges, which exposes the actual document content from all tabs when the includeTabsContent parameter is set to `true`. If `false` or unset, this field contains information about the first tab in the document.", alias="namedRanges")
    named_styles: Optional[NamedStyles] = Field(default=None, alias="namedStyles")
    positioned_objects: Optional[Dict[str, Any]] = Field(default=None, description="Output only. The positioned objects in the document, keyed by object ID. Legacy field: Instead, use Document.tabs.documentTab.positionedObjects, which exposes the actual document content from all tabs when the includeTabsContent parameter is set to `true`. If `false` or unset, this field contains information about the first tab in the document.", alias="positionedObjects")
    revision_id: Optional[StrictStr] = Field(default=None, description="Output only. The revision ID of the document. Can be used in update requests to specify which revision of a document to apply updates to and how the request should behave if the document has been edited since that revision. Only populated if the user has edit access to the document. The revision ID is not a sequential number but an opaque string. The format of the revision ID might change over time. A returned revision ID is only guaranteed to be valid for 24 hours after it has been returned and cannot be shared across users. If the revision ID is unchanged between calls, then the document has not changed. Conversely, a changed ID (for the same document and user) usually means the document has been updated. However, a changed ID can also be due to internal factors such as ID format changes.", alias="revisionId")
    suggested_document_style_changes: Optional[Dict[str, Any]] = Field(default=None, description="Output only. The suggested changes to the style of the document, keyed by suggestion ID. Legacy field: Instead, use Document.tabs.documentTab.suggestedDocumentStyleChanges, which exposes the actual document content from all tabs when the includeTabsContent parameter is set to `true`. If `false` or unset, this field contains information about the first tab in the document.", alias="suggestedDocumentStyleChanges")
    suggested_named_styles_changes: Optional[Dict[str, Any]] = Field(default=None, description="Output only. The suggested changes to the named styles of the document, keyed by suggestion ID. Legacy field: Instead, use Document.tabs.documentTab.suggestedNamedStylesChanges, which exposes the actual document content from all tabs when the includeTabsContent parameter is set to `true`. If `false` or unset, this field contains information about the first tab in the document.", alias="suggestedNamedStylesChanges")
    suggestions_view_mode: Optional[StrictStr] = Field(default=None, description="Output only. The suggestions view mode applied to the document. Note: When editing a document, changes must be based on a document with SUGGESTIONS_INLINE.", alias="suggestionsViewMode")
    tabs: Optional[List[Tab]] = Field(default=None, description="Tabs that are part of a document. Tabs can contain child tabs, a tab nested within another tab. Child tabs are represented by the Tab.childTabs field.")
    title: Optional[StrictStr] = Field(default=None, description="The title of the document.")
    __properties: ClassVar[List[str]] = ["body", "documentId", "documentStyle", "footers", "footnotes", "headers", "inlineObjects", "lists", "namedRanges", "namedStyles", "positionedObjects", "revisionId", "suggestedDocumentStyleChanges", "suggestedNamedStylesChanges", "suggestionsViewMode", "tabs", "title"]

    @field_validator('suggestions_view_mode')
    def suggestions_view_mode_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in set(['DEFAULT_FOR_CURRENT_ACCESS', 'SUGGESTIONS_INLINE', 'PREVIEW_SUGGESTIONS_ACCEPTED', 'PREVIEW_WITHOUT_SUGGESTIONS']):
            raise ValueError("must be one of enum values ('DEFAULT_FOR_CURRENT_ACCESS', 'SUGGESTIONS_INLINE', 'PREVIEW_SUGGESTIONS_ACCEPTED', 'PREVIEW_WITHOUT_SUGGESTIONS')")
        return value

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of Document from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of body
        if self.body:
            _dict['body'] = self.body.to_dict()
        # override the default output from pydantic by calling `to_dict()` of document_style
        if self.document_style:
            _dict['documentStyle'] = self.document_style.to_dict()
        # override the default output from pydantic by calling `to_dict()` of named_styles
        if self.named_styles:
            _dict['namedStyles'] = self.named_styles.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in tabs (list)
        _items = []
        if self.tabs:
            for _item_tabs in self.tabs:
                if _item_tabs:
                    _items.append(_item_tabs.to_dict())
            _dict['tabs'] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of Document from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "body": Body.from_dict(obj["body"]) if obj.get("body") is not None else None,
            "documentId": obj.get("documentId"),
            "documentStyle": DocumentStyle.from_dict(obj["documentStyle"]) if obj.get("documentStyle") is not None else None,
            "footers": obj.get("footers"),
            "footnotes": obj.get("footnotes"),
            "headers": obj.get("headers"),
            "inlineObjects": obj.get("inlineObjects"),
            "lists": obj.get("lists"),
            "namedRanges": obj.get("namedRanges"),
            "namedStyles": NamedStyles.from_dict(obj["namedStyles"]) if obj.get("namedStyles") is not None else None,
            "positionedObjects": obj.get("positionedObjects"),
            "revisionId": obj.get("revisionId"),
            "suggestedDocumentStyleChanges": obj.get("suggestedDocumentStyleChanges"),
            "suggestedNamedStylesChanges": obj.get("suggestedNamedStylesChanges"),
            "suggestionsViewMode": obj.get("suggestionsViewMode"),
            "tabs": [Tab.from_dict(_item) for _item in obj["tabs"]] if obj.get("tabs") is not None else None,
            "title": obj.get("title")
        })
        return _obj


